%% Problem: Sharp feature extraction at single scale
%
% Authors : Trung-Thien Tran
% Emal:       trung-thien.tran.1@ulaval.ca; 
%             tranthien9.vn@gmail.com
% Created : 30 Jan 2013
% From: 
% This implementation copyright Trung-Thien Tran 2013, vision.gel.ulaval.ca, 
% but is freely available to copy, use, or modify as long as this line is maintained
%% ----------------------------------------------------------------------------------%
% Modification author: Pawel Burdziakowski;
% email: pawel.burdziakowski@pg.edu.pl;
% created: 01.11.2020;
% Affiliation: Gdansk University Of Technology;
% note: Original code (author stated above) was modified;
% Tested: Matlab 2020a;
% The original code by Trung-Thien Tran was modified to fit new Matlab 2020a
% Toolboxes needed: 
%   Image Processing Toolbox
%   Computer Vision Toolbox
%   Statistics and Machine Learning Toolbox
%   for GPU processing: Parallel Computing Toolbox - GPU functions are
%   comented.
%
% If you find this code helpful you are to cite those two papers:
% 1:
% Burdziakowski, P.; Zakrzewska, A. 
% "A New Adaptive Method for the Extraction of Steel Design Structures from an Integrated Point Cloud.
% Sensors 2021, 21,
% 2:
% T. Tran, V. Cao, V. T. Nguyen, S. Ali and D. Laurendeau, 
% "Automatic method for sharp feature extraction from 3D data of man-made objects," 
% 2014 International Conference on Computer Graphics Theory and Applications (GRAPP), 2014, pp. 1-8.
% ----------------------------------------------------------------------------------%

function edge_out=feature_extract_neighbor_mod(ptCloud,k)

%da=gpuArray(ptCloud.Location); %uncomment it for GPU computing
da=ptCloud.Location; %uncomment it for CPU computing

[~,n]=size(da); 

%------------------------------------------------------------------
tic
[idx,D] = knnsearch(da,da(:,:),'k',k); %Eq 15
normals = pcnormals(ptCloud,k); %Eq 15
 
NN = D(:,2);
idx = idx';

X = da(:,1);
Y = da(:,2);
Z = da(:,3);

x = X(idx);
y = Y(idx);
z = Z(idx);

centroid = [mean(x,1);mean(y,1);mean(z,1)]'; %Eq 16
pdist = abs(dot((da-centroid),normals,2)); %Eq 17

disp (toc)

%%  Adaptive  threshold
%pdist=mat2gray(pdist); % normalize
%T = graythresh(pdist); %Otsu %ok
%T = adaptthresh(pdist, 'Statistic','gaussian'); % Gausian
%T = adaptthresh(pdist, 'Statistic','median'); % Median
%T = adaptthresh(pdist); %Bradley Mean
%pdist1=imbinarize(pdist,T); 

edge_out=pdist;


