%% ----------------------------------------------------------------------------------%
% Code for paper:
% Burdziakowski, P.; Zakrzewska, A. 
% "A New Adaptive Method for the Extraction of Steel Design Structures from an Integrated Point Cloud.
% Sensors 2021, 21,
% -----------------------------------------------------------------------------------%
% Code author: Pawel Burdziakowski;
% email: pawel.burdziakowski@pg.edu.pl;
% created: 01.11.2020;
% Affiliation: Gdansk University Of Technology;
% Tested: Matlab 2020a;
% -----------------------------------------------------------------------------------%
% If you find this code helpful you are to cite those two papers:
% 1:
% Burdziakowski, P.; Zakrzewska, A. 
% "A New Adaptive Method for the Extraction of Steel Design Structures from
% an Integrated Point Cloud"
% Sensors 2021, 21,
% 2:
% T. Tran, V. Cao, V. T. Nguyen, S. Ali and D. Laurendeau, 
% "Automatic method for sharp feature extraction from 3D data of man-made objects," 
% 2014 International Conference on Computer Graphics Theory and Applications (GRAPP), 2014, pp. 1-8.
% ----------------------------------------------------------------------------------%
clc
clear all
close all
warning off

%% Model loading
%name = 'pal-03.pcd'; %
name = 'box_1.ply'; %
%name = 'pal-segm.pcd'; %
%name = 'pal-segm-005.pcd'; %
%name = 'pal-segm-01.pcd'; %
%name = 'pal-segm-03.pcd'; %
%name = 'pal-segm-05.pcd'; %
%name = 'pal-segm-07.pcd'; %

%% Read point cloud
tic
ptCloud = pcread(name);

da=ptCloud.Location;
n=size(da,1);
disp ('Point Cloud Loaded')
disp (toc)
%% Multiple scale and sharp feature extraction
%k=8:4:128; % ok 31 scales
k=8:8:128; % Very OK
%k=4:2:64;

sz=size(k,2);
%M=gpuArray(zeros(sz,n)); %uncomment it for GPU computing
M=zeros(sz,n); %uncomment it for CPU computing

tic
for i=1:sz
    M(i,:)=feature_extract_neighbor_mod(ptCloud,k(i));
    disp(strcat('Iter-',int2str(i),'/',int2str(sz)))
end
disp(toc)

%T = adaptthresh(M, 'Statistic','gaussian'); % Gausian
%T = adaptthresh(M, 'Statistic','median'); % Median
T = adaptthresh(M); %Bradley Mean Adaprive Tresholding
M1=imbinarize(M,T); 

%% Sharp feeature extraction Eq 18 in the paper
A=[];
for i=1:n
    bool_value=find(M1(:,i)==1);
    if (size(bool_value,1)>=9*sz/10)  
        A=[A i];
    end        
end

% display and save results to PCD file

ptResult = pointCloud(da(A,:));
pcwrite(ptResult, strcat('result-',name));

pcshow(ptResult);
%pcshowpair(ptCloud,ptResult)
